<?php
/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
 
 class CRM_FormProcessor_BAO_FormProcessor extends CRM_FormProcessor_DAO_FormProcessor {
 	
	/**
   * Function to get values
   * 
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $formProcessor = new CRM_FormProcessor_BAO_FormProcessor();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $formProcessor->$key = $value;
        }
      }
    }
    $formProcessor->find();
    while ($formProcessor->fetch()) {
      $row = array();
      self::storeValues($formProcessor, $row); 
			$row['inputs'] = array_values(CRM_FormProcessor_BAO_FormProcessorInput::getValues(array('form_processor_id' => $formProcessor->id)));
			$row['actions'] = array_values(CRM_FormProcessor_BAO_FormProcessorAction::getValues(array('form_processor_id' => $formProcessor->id)));
			
      $result[$row['id']] = $row;
    }
    return $result;
  }
 	
	/**
   * Function to add or update a FormProcessor
   * 
   * @param array $params 
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a civirule rule');
    }

    if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessor', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessor', NULL, $params);
    }

    $formProcessor = new CRM_FormProcessor_BAO_FormProcessor();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $formProcessor->$key = $value;
      }
    }
    if (!isset($formProcessor->name) || empty($formProcessor->name)) {
      if (isset($formProcessor->title)) {
        $formProcessor->name = CRM_FormProcessor_BAO_FormProcessor::buildNameFromLabel($formProcessor->title);
      }
    }
    $formProcessor->save();
    self::storeValues($formProcessor, $result);

    if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessor', $formProcessor->id, $formProcessor);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessor', $formProcessor->id, $formProcessor);
    }

    return $result;
  }

	/**
   * Function to delete a FormProcessor with id
   * 
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor');
    }

    CRM_Utils_Hook::pre('delete', 'FormProcessor', $id, CRM_Core_DAO::$_nullArray);

		// First delete all inputs
		CRM_FormProcessor_BAO_FormProcessorInput::deleteWithFormProcessorId($id);

    $rule = new CRM_FormProcessor_BAO_FormProcessor();
    $rule->id = $id;
    $rule->delete();

    CRM_Utils_Hook::post('delete', 'FormProcessor', $id, CRM_Core_DAO::$_nullArray);

    return;
  }

	/**
   * Public function to generate name from label
   *
   * @param $label
   * @return string
   * @access public
   * @static
   */
  public static function buildNameFromLabel($label) {
    $labelParts = explode(' ', strtolower($label));
    $nameString = implode('_', $labelParts);
    return substr($nameString, 0, 80);
  }
	
	/**
	 * Returns whether the name is valid or not
	 * 
	 * @param string $name
	 * @param int $id optional
	 * @return bool
	 * @static
	 */
	public static function isNameValid($name, $id=null) {
		$sql = "SELECT COUNT(*) FROM `civicrm_form_processor` WHERE `name` = %1";
		$params[1] = array($name, 'String');
		if ($id) {
			$sql .= " AND `id` != %2";
			$params[2] = array($id, 'Integer');
		}
		$count = CRM_Core_DAO::singleValueQuery($sql, $params);
		return ($count > 0) ? false : true;
	}
	
 }
