<?php

use CRM_FormProcessor_ExtensionUtil as E;

/**
 * @author Jaap Jansma (CiviCooP) <jaap.jansma@civicoop.org>
 * @license http://www.gnu.org/licenses/agpl-3.0.html
 */
class CRM_FormProcessor_BAO_FormProcessorAction extends CRM_FormProcessor_DAO_FormProcessorAction {
  
	 /**
   * Function to get values
   * 
   * @return array $result found rows with data
   * @access public
   * @static
   */
  public static function getValues($params) {
    $result = array();
    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    if (!empty($params)) {
      $fields = self::fields();
      foreach ($params as $key => $value) {
        if (isset($fields[$key])) {
          $action->$key = $value;
        }
      }
    }
		$action->orderBy('weight');
    $action->find();
    while ($action->fetch()) {
      $row = array();
      self::storeValues($action, $row);
      if (!empty($row['form_processor_id'])) {
      	$provider = form_processor_get_action_provider();
				$action_type = $provider->getActionByName($name);
				
				if ($action_type) {
					$configuration = $action_type->getDefaultConfiguration();
					if (isset($row['configuration']) && is_string($row['configuration'])) {
						$row['configuration'] = json_decode($row['configuration'], true);
					}
					if (empty($row['configuration']) || !is_array($row['configuration'])) {
						$row['configuration'] = array();
					}
					foreach($row['configuration'] as $name => $value) {
						$configuration->set($name, $value);
					}
					$action_type->setConfiguration($configuration);
					
					$row['type'] = $action_type->toArray();
				}
        $result[$row['id']] = $row;
      } else {
        //invalid input because no there is no form processor
        CRM_FormProcessor_BAO_FormProcessorAction::deleteWithId($row['id']);
      }
    }
    return $result;
  }

  /**
   * Function to add or update form processor action
   * 
   * @param array $params 
   * @return array $result
   * @access public
   * @throws Exception when params is empty
   * @static
   */
  public static function add($params) {
    $result = array();
    if (empty($params)) {
      throw new Exception('Params can not be empty when adding or updating a form processor action');
    }
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::pre('edit', 'FormProcessorAction', $params['id'], $params);
    }
    else {
      CRM_Utils_Hook::pre('create', 'FormProcessorAction', NULL, $params);
    }
		
    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    $fields = self::fields();
    foreach ($params as $key => $value) {
      if (isset($fields[$key])) {
        $action->$key = $value;
      }
    }
		if (is_array($action->configuration)) {
			$action->configuration = json_encode($action->configuration);
		}
    $action->save();
    self::storeValues($action, $result);
		
		if (!empty($params['id'])) {
      CRM_Utils_Hook::post('edit', 'FormProcessorAction', $action->id, $action);
    }
    else {
      CRM_Utils_Hook::post('create', 'FormProcessorAction', $action->id, $action);
    }
		
    return $result;
  }

  /**
   * Function to delete a form processor action with id
   * 
   * @param int $id
   * @throws Exception when $id is empty
   * @access public
   * @static
   */
  public static function deleteWithId($id) {
    if (empty($id)) {
      throw new Exception('id can not be empty when attempting to delete a form processor action');
    }
		
		CRM_Utils_Hook::pre('delete', 'FormProcessorAction', $id, CRM_Core_DAO::$_nullArray);
		
    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    $action->id = $id;
    if ($action->find(true)) {
      $action->delete();
    }
		
		CRM_Utils_Hook::post('delete', 'FormProcessorAction', $id, CRM_Core_DAO::$_nullArray);
		
    return;
  }

	/**
   * Function to delete all actions with a form processor id
   *
   * @param int $formProcessorId
   * @access public
   * @static
   */
  public static function deleteWithFormProcessorId($formProcessorId) {
    $action = new CRM_FormProcessor_BAO_FormProcessorAction();
    $action->form_processor_id = $formProcessorId;
    $action->find(FALSE);
    while ($input->fetch()) {
      CRM_Utils_Hook::pre('delete', 'FormProcessorAction', $action->id, CRM_Core_DAO::$_nullArray);
		
	    $action->id = $id;
      $action->delete();
			
			CRM_Utils_Hook::post('delete', 'FormProcessorAction', $action->id, CRM_Core_DAO::$_nullArray);
    }
  }
	
	
}